@@ -56,10 +56,12 @@
 
 QT_BEGIN_NAMESPACE
 
+Q_LOGGING_CATEGORY(lcItemManagement, "qt.quick.controls.control.itemmanagement")
+
 /*!
     \qmltype Control
     \inherits Item
-    \instantiates QQuickControl
+//!     \instantiates QQuickControl
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \brief Abstract base type providing functionality common to all controls.
@@ -122,10 +124,20 @@ QT_BEGIN_NAMESPACE
     \section1 Event Handling
 
     All controls, except non-interactive indicators, do not let clicks and
-    touches through to items below them. For example, if \l Pane is used as the
-    \l {ApplicationWindow::}{header} or \l {ApplicationWindow::}{footer} of
-    \l ApplicationWindow, items underneath it will not get mouse or touch
-    events.
+    touches through to items below them. For example, the \c console.log()
+    call in the example below will never be executed when clicking on the
+    Pane, because the \l MouseArea is below it in the scene:
+
+    \code
+    MouseArea {
+        anchors.fill: parent
+        onClicked: console.log("MouseArea was clicked")
+
+        Pane {
+            anchors.fill: parent
+        }
+    }
+    \endcode
 
     \sa ApplicationWindow, Container
 */
@@ -168,6 +180,12 @@ bool QQuickControlPrivate::acceptTouch(const QTouchEvent::TouchPoint &point)
         return true;
     }
 
+    // If the control is on a Flickable that has a pressDelay, then the press is never
+    // sent as a touch event, therefore we need to check for this case.
+    if (touchId == -1 && pressWasTouch && point.state() == Qt::TouchPointReleased &&
+        point.pos() == previousPressPos) {
+        return true;
+    }
     return false;
 }
 #endif
@@ -203,6 +221,8 @@ void QQuickControlPrivate::handleRelease(const QPointF &)
     if ((focusPolicy & Qt::ClickFocus) == Qt::ClickFocus && QGuiApplication::styleHints()->setFocusOnTouchRelease())
         setActiveFocus(q, Qt::MouseFocusReason);
     touchId = -1;
+    pressWasTouch = false;
+    previousPressPos = QPointF();
 }
 
 void QQuickControlPrivate::handleUngrab()
@@ -410,10 +430,10 @@ void QQuickControlPrivate::setContentItem_helper(QQuickItem *item, bool notify)
 
     contentItem = item;
     q->contentItemChange(item, oldContentItem);
-    delete oldContentItem;
+    QQuickControlPrivate::hideOldItem(oldContentItem);
 
     if (item) {
-        connect(contentItem, &QQuickItem::baselineOffsetChanged, this, &QQuickControlPrivate::updateBaselineOffset);
+        connect(contentItem.data(), &QQuickItem::baselineOffsetChanged, this, &QQuickControlPrivate::updateBaselineOffset);
         if (!item->parentItem())
             item->setParentItem(q);
         if (componentComplete)
@@ -539,7 +559,13 @@ void QQuickControlPrivate::inheritFont(const QFont &font)
     parentFont.resolve(extra.isAllocated() ? extra->requestedFont.resolve() | font.resolve() : font.resolve());
 
     const QFont defaultFont = q->defaultFont();
-    const QFont resolvedFont = parentFont.resolve(defaultFont);
+    QFont resolvedFont = parentFont.resolve(defaultFont);
+    // Since resolving the font will put the family() into the
+    // families() list if it is empty then we need to unset it
+    // so it does not act as if the font has changed (when it
+    // has not actually changed)
+    if (defaultFont.families().isEmpty())
+        resolvedFont.setFamilies(QStringList());
 
     setFont_helper(resolvedFont);
 }
@@ -822,6 +848,24 @@ void QQuickControlPrivate::executeBackground(bool complete)
         quickCompleteDeferred(q, backgroundName(), background);
 }
 
+void QQuickControlPrivate::hideOldItem(QQuickItem *item)
+{
+    if (!item)
+        return;
+
+    qCDebug(lcItemManagement) << "hiding old item" << item;
+
+    item->setVisible(false);
+    item->setParentItem(nullptr);
+
+#if QT_CONFIG(accessibility)
+    // Remove the item from the accessibility tree.
+    QQuickAccessibleAttached *accessible = accessibleAttached(item);
+    if (accessible)
+        accessible->setIgnored(true);
+#endif
+}
+
 void QQuickControlPrivate::updateBaselineOffset()
 {
     Q_Q(QQuickControl);
@@ -933,6 +977,7 @@ void QQuickControl::itemChange(QQuickItem::ItemChange change, const QQuickItem::
     switch (change) {
     case ItemEnabledHasChanged:
         emit paletteChanged();
+        enabledChange();
         break;
     case ItemVisibleHasChanged:
 #if QT_CONFIG(quicktemplates2_hover)
@@ -1459,9 +1504,9 @@ void QQuickControl::setHovered(bool hovered)
     Setting this property propagates the value to all child controls that do not have
     \c hoverEnabled explicitly set.
 
-    You can also enable or disable hover effects for all Qt Quick Controls 2 applications
+    You can also enable or disable hover effects for all Qt Quick Controls applications
     by setting the \c QT_QUICK_CONTROLS_HOVER_ENABLED \l {Supported Environment Variables
-    in Qt Quick Controls 2}{environment variable}.
+    in Qt Quick Controls}{environment variable}.
 
     \sa hovered
 */
@@ -1573,7 +1618,7 @@ void QQuickControl::setBackground(QQuickItem *background)
     }
 
     d->removeImplicitSizeListener(d->background, QQuickControlPrivate::ImplicitSizeChanges | QQuickItemPrivate::Geometry);
-    delete d->background;
+    QQuickControlPrivate::hideOldItem(d->background);
     d->background = background;
 
     if (background) {
@@ -1615,8 +1660,11 @@ void QQuickControl::setBackground(QQuickItem *background)
     }
     \endcode
 
-    \note The content item is automatically resized to fit within the
-    \l padding of the control.
+    \note The content item is automatically positioned and resized to fit
+    within the \l padding of the control. Bindings to the
+    \l[QtQuick]{Item::}{x}, \l[QtQuick]{Item::}{y},
+    \l[QtQuick]{Item::}{width}, and \l[QtQuick]{Item::}{height}
+    properties of the contentItem are not respected.
 
     \note Most controls use the implicit size of the content item to calculate
     the implicit size of the control itself. If you replace the content item
@@ -2083,6 +2131,10 @@ void QQuickControl::mousePressEvent(QMouseEvent *event)
 {
     Q_D(QQuickControl);
     d->handlePress(event->localPos());
+    if (event->source() == Qt::MouseEventSynthesizedByQt) {
+        d->pressWasTouch = true;
+        d->previousPressPos = event->localPos();
+    }
     event->accept();
 }
 
@@ -2174,6 +2226,10 @@ void QQuickControl::geometryChanged(const QRectF &newGeometry, const QRectF &old
         emit availableHeightChanged();
 }
 
+void QQuickControl::enabledChange()
+{
+}
+
 void QQuickControl::fontChange(const QFont &newFont, const QFont &oldFont)
 {
     Q_UNUSED(newFont);
@@ -2258,11 +2314,13 @@ QString QQuickControl::accessibleName() const
     return QString();
 }
 
-void QQuickControl::setAccessibleName(const QString &name)
+void QQuickControl::maybeSetAccessibleName(const QString &name)
 {
 #if QT_CONFIG(accessibility)
-    if (QQuickAccessibleAttached *accessibleAttached = QQuickControlPrivate::accessibleAttached(this))
-        accessibleAttached->setName(name);
+    if (QQuickAccessibleAttached *accessibleAttached = QQuickControlPrivate::accessibleAttached(this)) {
+        if (!accessibleAttached->wasNameExplicitlySet())
+            accessibleAttached->setNameImplicitly(name);
+    }
 #else
     Q_UNUSED(name)
 #endif
